package classinfo;

public
class Utf {
  private char[] value;
  private int utflen;
  private byte[] utfValue;

  public Utf(String str) {
    int len = str.length();

    value = new char[len];
    str.getChars(0, len, value, 0);

    this.makeUtfForm();
  }

  public Utf(char[] chars) {
    System.arraycopy(chars, 0, value, 0, chars.length);

    this.makeUtfForm();
  }

  public Utf(byte[] utfBytes) {
    this(utfBytes, 0, utfBytes.length);
  }

  public Utf(byte[] utfBytes, int offset, int length) {
    utflen = length;
    utfValue = new byte[length];
    System.arraycopy(utfBytes, offset, utfValue, 0, utflen);

    this.makeUnicodeForm();
  }


  public String toString() {
    return new String(value);
  }

  public int length() {
    return value.length;
  }

  public int utfLength() {
    return this.utflen;
  }

  public void getBytes(byte[] dst) {
    System.arraycopy(utfValue, 0, dst, 0, utflen);
  }


  private void makeUtfForm() {
    int length = value.length;
    this.utflen = 0;

    for (int i = 0; i < length; i++) {
      int c = value[i];
      if ((c >= 0x0001) && (c <= 0x007f)) {
	utflen++;
      }
      else if (c > 0x7ff) {
	utflen += 3;
      }
      else {
	utflen += 2;
      }
    }


    this.utfValue = new byte[this.utflen];

    int utfCount = 0;
    for (int i = 0; i < length; i++) {
      int c = value[i];
      if ((c >= 0x0001) && (c <= 0x007f)) {
	this.utfValue[utfCount++] = (byte)c;
      }
      else if (c == 0) {
	this.utfValue[utfCount++] = (byte)0xc0;
	this.utfValue[utfCount++] = (byte)0x80;
      }
      else if (c <= 0x07ff) {
	this.utfValue[utfCount++] = (byte)(0xc0 | ((c >> 6) & 0x1f));
	this.utfValue[utfCount++] = (byte)(0x80 | (c & 0x3f));
      }
      else if (c > 0x07ff) {
	this.utfValue[utfCount++] = (byte)(0xe0 | ((c >> 12) & 0x0f));
	this.utfValue[utfCount++] = (byte)(0x80 | ((c >> 6) & 0x3f));
	this.utfValue[utfCount++] = (byte)(0x80 | (c & 0x3f));
      }
    }
  }


  private void makeUnicodeForm() {
    char[] unicodeForm = new char[this.utflen];

    int utfCount = 0;
    int unicodeCount = 0;
    while (utfCount < utflen) {
      int c = 0;
      int b = utfValue[utfCount++];
      if (b <= 0x7f) {
	unicodeForm[unicodeCount++] = (char)b;
      }
      else if (b <= 0xdf) {
	c |= (b & 0x1f) << 6;
	b = utfValue[utfCount++];
	c |= b & 0x3f;
	unicodeForm[unicodeCount++] = (char)c;
      }
      else if (b <= 0xef) {
	c |= (b & 0x0f) << 12;
	b = utfValue[utfCount++];
	c |= (b & 0x3f) << 6;
	b = utfValue[utfCount++];
	c |= b & 0x3f;
	unicodeForm[unicodeCount++] = (char)c;
      }
      else {	// invalid
	unicodeForm[unicodeCount++] = 0;
      }
    }

    this.value = new char[unicodeCount];
    System.arraycopy(unicodeForm, 0, this.value, 0, unicodeCount);
  }
}
