package classinfo.attribute;

import java.io.DataOutputStream;
import java.io.IOException;
import java.util.Vector;
import classinfo.*;


public
class InnerClasses extends AttributeInfo {
  public static final String ATTRIBUTE_NAME = "InnerClasses";

  static private class ClassEntry {
    ConstantPoolReference innerClass;
    ConstantPoolReference outerClass;
    ConstantPoolReference innerName;
    short innerFlags;
  }

  ClassEntry[] classes;
  int nClasses;

  public InnerClasses(ConstantPoolReference nameIndex, int length,
			BufManager bufm, Vector CPTable)
	throws IOException {
    super(nameIndex, length);

    bufm.clearBuffer();
    bufm.advance(2);

    // number of classes
    nClasses = bufm.parseInt(0, 2);

    // classes
    classes = new ClassEntry[nClasses];
    for (int i = 0; i < nClasses; i++) {
      bufm.clearBuffer();
      bufm.advance(8);

      ClassEntry entry = new ClassEntry();
      classes[i] = entry;
      entry.innerClass =
	new ConstantPoolReference(bufm.parseInt(0, 2), CPTable);
      entry.outerClass =
	new ConstantPoolReference(bufm.parseInt(2, 2), CPTable);
      entry.innerName =
	new ConstantPoolReference(bufm.parseInt(4, 2), CPTable);
      entry.innerFlags = (short)bufm.parseInt(6, 2);
    }
  }


  public void write(DataOutputStream out)
	throws IOException {
    out.writeShort((short)nClasses);
    for (int i = 0; i < nClasses; i++) {
      ClassEntry entry = classes[i];
      out.writeShort((short)entry.innerClass.index());
      out.writeShort((short)entry.outerClass.index());
      out.writeShort((short)entry.innerName.index());
      out.writeShort(entry.innerFlags);
    }
  }


  public String toString() { return toString(""); }

  public String toString(String indent) {
    StringBuffer buf = new StringBuffer(AttributeInfo.STRING_BUFFER_SIZE);

    buf.append(super.toString(indent));

    if (indent == null)  indent = "\t";
    else  indent += "\t";

    buf.append(indent + "Number of classes: " + nClasses + "\n");
    for (int i = 0; i < nClasses; i++) {
      ClassEntry entry = classes[i];
      buf.append(indent + "\tinner class: " +
		entry.innerClass.toString() + "\n");
      buf.append(indent + "\touter class: " +
		entry.outerClass.toString() + "\n");
      buf.append(indent + "\tinner name: " +
		entry.innerName.toString() + "\n");
      buf.append(indent + "\tinner class access flags: " +
		AccessFlags.toString(entry.innerFlags) + "\n");
    }

    return buf.toString();
  }
}
