package classinfo.attribute;

import java.io.DataOutputStream;
import java.io.IOException;
import java.util.Vector;
import classinfo.*;

public
class LineNumberTable extends AttributeInfo {
  public static final String ATTRIBUTE_NAME = "LineNumberTable";

  int tableLength;	// 2 bytes
  int[] startPC;		// each entry is 2 bytes
  int[] lineNumber;		// each entry is 2 bytes

  public LineNumberTable(ConstantPoolReference nameIndex, int length,
			BufManager bufm, Vector CPTable)
	throws IOException {
    super(nameIndex, length);

    bufm.clearBuffer();
    bufm.advance(2);

    // line number table length
    tableLength = bufm.parseInt(0, 2);

    // line number table
    startPC = new int[tableLength];
    lineNumber = new int[tableLength];

    for (int i = 0; i < tableLength; i++) {
      bufm.clearBuffer();
      bufm.advance(4);

      startPC[i] = bufm.parseInt(0, 2);
      lineNumber[i] = bufm.parseInt(2, 2);
    }
  }


  public void write(DataOutputStream out)
	throws java.io.IOException {
    out.writeShort((short)tableLength);
    for (int i = 0; i < tableLength; i++) {
      out.writeShort((short)startPC[i]);
      out.writeShort((short)lineNumber[i]);
    }
  }


  public String toString() { return toString(""); }

  public String toString(String indent) {
    StringBuffer buf = new StringBuffer(AttributeInfo.STRING_BUFFER_SIZE);

    buf.append(super.toString(indent));

    indent = indent + "\t";
    buf.append(indent + "Line number table length: " +
					tableLength + "\n");
    buf.append(indent + "Line number table:\n");

    for (int i = 0; i < tableLength; i++) {
      buf.append(indent + "\tstart pc, line number: " +
			startPC[i] + ", " + lineNumber[i] + "\n");
    }

    return buf.toString();
  }
}
